@@ -9,7 +9,8 @@ See add_append_functionality_to_store_cls docs for examples.
 
 import time
 import types
-from typing import Callable, Optional
+from typing import Optional
+from collections.abc import Callable
 
 from dol.trans import store_decorator
 from dol.util import exhaust
@@ -49,9 +50,9 @@ def define_extend_as_seq_of_appends(obj):
     [1, 2, 3, 10, 20]
 
     """
-    assert hasattr(
-        obj, 'append'
-    ), f'Your object needs to have an append method! Object was: {obj}'
+    assert hasattr(obj, "append"), (
+        f"Your object needs to have an append method! Object was: {obj}"
+    )
 
     def extend(self, items):
         for item in items:
@@ -106,6 +107,39 @@ class mk_item2kv_for:
     (('go', 'let'), {'I': 'it'})
     """
 
+    @staticmethod
+    def attr(attr_name):
+        """Make item2kv from an attribute name (the value will be the item itself).
+
+        Args:
+            attr_name: The attribute name to use as the key
+        Returns: an item -> (key, val) function
+
+        >>> ref_getter =mk_item2kv_for.attr("ref")
+        >>> from collections import namedtuple
+        >>> A = namedtuple('A', ['ref'])
+        >>> a = A(ref='some_ref')
+        >>> ref_getter(a)
+        ('some_ref', A(ref='some_ref'))
+
+        """
+
+        def item2kv(item):
+            return getattr(item, attr_name), item
+
+        return item2kv
+
+    @staticmethod
+    def kv_pairs():
+        """
+        Essentially, the identity. Is used when the items are already (key, val) pairs.
+        """
+
+        def item2kv(item):
+            return item
+
+        return item2kv
+
     @staticmethod
     def item_to_key(item2key):
         """Make item2kv from a item2key function (the value will be the item itself).
@@ -176,7 +210,7 @@ class mk_item2kv_for:
         return item2kv
 
     @staticmethod
-    def utc_key(offset_s=0, factor=1, *, time_postproc: Optional[Callable] = None):
+    def utc_key(offset_s=0, factor=1, *, time_postproc: Callable | None = None):
         """Make an item2kv function that uses the current time as the key, and the unchanged item as a value.
         The offset_s, which is added to the output key, can be used, for example, to align to another system's clock,
         or to get a more accurate timestamp of an event.
@@ -218,6 +252,45 @@ class mk_item2kv_for:
 
         return item2kv
 
+    @staticmethod
+    def uuid_key(hex=True):
+        """Make an item2kv function that uses a uuid hex as the key.
+        This uses uuid.uuid1() to generate the uuid, so it is not cryptographically
+        secure, and it may not be unique if you generate more than 10k uuids per second.
+        One advantage though, is that the uuid is time-based, so it can be used to sort
+        the keys in the order they were IDed.
+
+        Returns: an item -> (uuid, item) function
+
+        >>> import uuid
+        >>> item2key = mk_item2kv_for.uuid_key()
+        >>> k, v = item2key('some data')
+        >>> v
+        'some data'
+        >>> isinstance(k, str)
+        True
+        >>> k  # doctest: +SKIP
+        '0d8a9930aaf411ee9f605e03a02258c9'
+        >>> item2key = mk_item2kv_for.uuid_key(hex=False)
+        >>> k, v = item2key('some data')
+        >>> isinstance(k, uuid.UUID)
+        True
+
+        """
+        import uuid
+
+        if hex:
+
+            def item2kv(item):
+                return uuid.uuid1().hex, item
+
+        else:
+
+            def item2kv(item):
+                return uuid.uuid1(), item
+
+        return item2kv
+
     @staticmethod
     def item_to_key_params_and_val(item_to_key_params_and_val, key_str_format):
         """Make item2kv from a function that produces key_params and val,
@@ -380,9 +453,9 @@ def appendable(store_cls=None, *, item2kv, return_keys=False):
         exhaust(gen)
 
     return type(
-        'Appendable' + store_cls.__name__,
+        "Appendable" + store_cls.__name__,
         (store_cls,),
-        {'append': append, 'extend': extend},
+        {"append": append, "extend": extend},
     )
 
 
@@ -391,9 +464,89 @@ add_append_functionality_to_store_cls = appendable  # for back compatibility
 appendable.mk_item2kv_for = mk_item2kv_for  # adding as attribute for convenient access
 
 from collections.abc import Sequence
-from typing import Iterable, Optional
+from typing import Optional
+from collections.abc import Iterable
 
-NotAVal = type('NotAVal', (), {})()  # singleton instance to distinguish from None
+NotAVal = type("NotAVal", (), {})()  # singleton instance to distinguish from None
+
+
+from collections.abc import MutableMapping
+from functools import partial
+from operator import add
+
+
+def read_add_write(store, key, iterable, add_iterables=add):
+    """Retrieves"""
+    if key in store:
+        store[key] = add_iterables(store[key], iterable)
+    else:
+        store[key] = iterable
+
+
+class Extender:
+    """Extends a value in a store.
+
+    The value in the store (if it exists) must be an iterable.
+    The value to extend must also be an iterable.
+
+    Unless a different ``extend_store_value`` function is given,
+    the sum of the two iterables must be an iterable.
+
+    The default ``extend_store_value`` is such that if the key is not in the store,
+    the value is simply written in the store.
+
+    The default ``append_method`` is ``None``, which means that the ``append`` method
+    is not defined. If you want to define it, you can pass a function that takes
+    the ``Extender`` instance as first argument, and the object to append as second
+    argument. The ``append`` method will then be defined as a partial of this function
+    with the ``Extender`` instance as first argument.
+
+    >>> store = {'a': 'pple'}
+    >>> # test normal extend
+    >>> a_extender = Extender(store, 'a')
+    >>> a_extender.extend('sauce')
+    >>> store
+    {'a': 'pplesauce'}
+    >>> # test creation (when key is not in store)
+    >>> b_extender = Extender(store, 'b')
+    >>> b_extender.extend('anana')
+    >>> store
+    {'a': 'pplesauce', 'b': 'anana'}
+    >>> # you can use the += operator too
+    >>> b_extender += ' split'
+    >>> store
+    {'a': 'pplesauce', 'b': 'anana split'}
+
+    """
+
+    def __init__(
+        self,
+        store: MutableMapping,
+        key,
+        *,
+        extend_store_value=read_add_write,
+        append_method=None,
+    ):
+        self.store = store
+        self.key = key
+        self.extend_store_value = extend_store_value
+
+        # Note: Not sure this is a good idea.
+        # Note:   I'm not documenting it or testing it until I let class mature.
+        # Note: Yes, I tried making this a method of the class, but it became ugly.
+        if append_method is not None:
+            self.append = partial(append_method, self)
+
+    def extend(self, iterable):
+        """Extend the iterable stored in"""
+        return self.extend_store_value(self.store, self.key, iterable)
+
+    __iadd__ = extend  # Note: Better to forward dunders to non-dunder-methods
+
+    # TODO: Should we even have this? Is it violating the purity of the class?
+    @property
+    def value(self):
+        return self.store[self.key]
 
 
 #
@@ -491,14 +644,14 @@ class FirstAppendOnly(Sequence):
 
     def __getitem__(self, k):
         if len(self) == 0:
-            raise IndexError(f'There are no items in this {self.__class__} instance')
+            raise IndexError(f"There are no items in this {self.__class__} instance")
         elif k == 0:
             return self.val
         elif isinstance(k, slice):
             return [self.val][k]
         else:
             raise IndexError(
-                f'A {self.__class__} instance can only have one value, or none at all.'
+                f"A {self.__class__} instance can only have one value, or none at all."
             )
 
     # @staticmethod
