@@ -34,44 +34,51 @@ from collections.abc import (
     ItemsView as BaseItemsView,
     Set,
 )
-from typing import Any, Iterable, Tuple, Callable, Union, Optional
+from typing import Any, Tuple, Union, Optional
+from collections.abc import Iterable, Callable
 
-from dol.util import wraps, _disabled_clear_method
+from dol.util import (
+    wraps,
+    _disabled_clear_method,
+    identity_func,
+    static_identity_method,
+    Key,
+    Val,
+    Id,
+    Data,
+    Item,
+    KeyIter,
+    ValIter,
+    ItemIter,
+    is_unbound_method,
+    is_classmethod,
+)
 from dol.signatures import Sig
 
-Key = Any
-Val = Any
-Id = Any
-Data = Any
-Item = Tuple[Key, Val]
-KeyIter = Iterable[Key]
-ValIter = Iterable[Val]
-ItemIter = Iterable[Item]
-
 
 class AttrNames:
-    CollectionABC = {'__len__', '__iter__', '__contains__'}
+    CollectionABC = {"__len__", "__iter__", "__contains__"}
     Mapping = CollectionABC | {
-        'keys',
-        'get',
-        'items',
-        '__reversed__',
-        'values',
-        '__getitem__',
+        "keys",
+        "get",
+        "items",
+        "__reversed__",
+        "values",
+        "__getitem__",
     }
     MutableMapping = Mapping | {
-        'setdefault',
-        'pop',
-        'popitem',
-        'clear',
-        'update',
-        '__delitem__',
-        '__setitem__',
+        "setdefault",
+        "pop",
+        "popitem",
+        "clear",
+        "update",
+        "__delitem__",
+        "__setitem__",
     }
 
-    Collection = CollectionABC | {'head'}
-    KvReader = (Mapping | {'head'}) - {'__reversed__'}
-    KvPersister = (MutableMapping | {'head'}) - {'__reversed__'} - {'clear'}
+    Collection = CollectionABC | {"head"}
+    KvReader = (Mapping | {"head"}) - {"__reversed__"}
+    KvPersister = (MutableMapping | {"head"}) - {"__reversed__"} - {"clear"}
 
 
 # TODO: Consider using ContainmentChecker and Sizer attributes which dunders would
@@ -107,7 +114,7 @@ class Collection(CollectionABC):
         return count
 
     def head(self):
-        if hasattr(self, 'items'):
+        if hasattr(self, "items"):
             return next(iter(self.items()))
         else:
             return next(iter(self))
@@ -169,14 +176,14 @@ class KvReader(MappingViewMixin, Collection, Mapping):
         .. code-block:: python
 
             reversed = list(self)[::-1]
-        
+
 
         If the keys are comparable, therefore sortable, another natural option would be:
-        
+
         .. code-block:: python
 
             reversed = sorted(self)[::-1]
-        
+
         """
         raise NotImplementedError(__doc__)
 
@@ -199,7 +206,7 @@ class KvPersister(KvReader, MutableMapping):
 
         s = SomeKvPersister()
         s['a']['b'] = 3
-    
+
     If `s` is a dict, this would have the effect of adding a ('b', 3) item under 'a'.
     But in the general case, this might
     - fail, because the `s['a']` doesn't support sub-scripting (doesn't have a `__getitem__`)
@@ -232,15 +239,6 @@ class KvPersister(KvReader, MutableMapping):
 Persister = KvPersister  # alias for back-compatibility
 
 
-# TODO: Make identity_func "identifiable". If we use the following one, we can use == to detect it's use,
-# TODO: ... but there may be a way to annotate, register, or type any identity function so it can be detected.
-def identity_func(x):
-    return x
-
-
-static_identity_method = staticmethod(identity_func)
-
-
 class NoSuchItem:
     pass
 
@@ -259,7 +257,22 @@ class DelegatedAttribute:
         if instance is None:
             # return getattr(getattr(owner, self.delegate_name), self.attr_name)
             # return getattr(owner, self.attr_name, None)
-            return self  # .__wrapped__  # TODO: The __wrapped__ would make it hard to debug
+
+            # TODO: Would just return self or self.__wrapped__ here, but
+            #   self.__wrapped__ would make it hard to debug and
+            #   self would fail with unbound methods (why?)
+            #   So doing a check here, but would like to find a better solution.
+            wrapped_self = getattr(self, "__wrapped__", None)
+            if is_classmethod(wrapped_self) or is_unbound_method(wrapped_self):
+                return wrapped_self
+            else:
+                return self
+
+            # wrapped_self = getattr(self, '__wrapped__', None)
+            # if not is_classmethod(wrapped_self):
+            #     return self
+            # else:
+            #     return wrapped_self
         else:
             # i.e. return instance.delegate.attr
             return getattr(getattr(instance, self.delegate_name), self.attr_name)
@@ -277,8 +290,8 @@ Decorator = Callable[[Callable], Any]  # TODO: Look up typing protocols
 
 def delegate_to(
     wrapped: type,
-    class_trans: Optional[Callable] = None,
-    delegation_attr: str = 'store',
+    class_trans: Callable | None = None,
+    delegation_attr: str = "store",
     include=frozenset(),
     ignore=frozenset(),
 ) -> Decorator:
@@ -304,9 +317,9 @@ def delegate_to(
             def __init__(self, *args, **kwargs):
                 delegate = wrapped(*args, **kwargs)
                 super().__init__(delegate)
-                assert isinstance(
-                    getattr(self, delegation_attr, None), wrapped
-                ), f'The wrapper instance has no (expected) {delegation_attr!r} attribute'
+                assert isinstance(getattr(self, delegation_attr, None), wrapped), (
+                    f"The wrapper instance has no (expected) {delegation_attr!r} attribute"
+                )
 
             def __reduce__(self):
                 return (
@@ -323,9 +336,15 @@ def delegate_to(
         )  # don't bother adding attributes that the class already has
         # set all the attributes
         for attr in attrs:
+            if attr == "__provides__":  # TODO: Hack. Find better solution.
+                # This is because __provides__ happened to be in wrapper_cls but not
+                # in wrapped.
+                # Happened at some point with `from sqldol import SqlRowsReader``
+                continue
             wrapped_attr = getattr(wrapped, attr)
             delegated_attribute = update_wrapper(
-                wrapper=DelegatedAttribute(delegation_attr, attr), wrapped=wrapped_attr,
+                wrapper=DelegatedAttribute(delegation_attr, attr),
+                wrapped=wrapped_attr,
             )
             setattr(Wrap, attr, delegated_attribute)
 
@@ -346,9 +365,9 @@ def wrapped_delegator_reconstruct(wrapped_cls, wrapped, class_trans, delegation_
 
 def delegator_wrap(
     delegator: Callable,
-    obj: Union[type, Any],
+    obj: type | Any,
     class_trans=None,
-    delegation_attr: str = 'store',
+    delegation_attr: str = "store",
 ):
     """Wrap a ``obj`` (type or instance) with ``delegator``.
 
@@ -424,7 +443,6 @@ def delegator_wrap(
     """
     if isinstance(obj, type):
         if isinstance(delegator, type):
-
             type_decorator = delegate_to(
                 obj, class_trans=class_trans, delegation_attr=delegation_attr
             )
@@ -559,7 +577,7 @@ class Store(KvPersister):
 
     """
 
-    _state_attrs = ['store', '_class_wrapper']
+    _state_attrs = ["store", "_class_wrapper"]
     # __slots__ = ('_id_of_key', '_key_of_id', '_data_of_obj', '_obj_of_data')
 
     def __init__(self, store=dict):
@@ -570,13 +588,13 @@ class Store(KvPersister):
 
         self.store = store
 
-        if hasattr(self.store, 'KeysView'):
+        if hasattr(self.store, "KeysView"):
             self.KeysView = self.store.KeysView
 
-        if hasattr(self.store, 'ValuesView'):
+        if hasattr(self.store, "ValuesView"):
             self.ValuesView = self.store.ValuesView
 
-        if hasattr(self.store, 'ItemsView'):
+        if hasattr(self.store, "ItemsView"):
             self.ItemsView = self.store.ItemsView
 
     _id_of_key = static_identity_method
@@ -590,13 +608,13 @@ class Store(KvPersister):
         KeyError,
     )  # another option: (KeyError, FileNotFoundError)
 
-    wrap = classmethod(partial(delegator_wrap, delegation_attr='store'))
+    wrap = classmethod(partial(delegator_wrap, delegation_attr="store"))
 
     def __getattr__(self, attr):
         """Delegate method to wrapped store if not part of wrapper store methods"""
         # Instead of return getattr(self.store, attr), doing the following
         # because self.store had problems with pickling
-        return getattr(object.__getattribute__(self, 'store'), attr)
+        return getattr(object.__getattribute__(self, "store"), attr)
 
     def __dir__(self):
         return list(
@@ -615,13 +633,13 @@ class Store(KvPersister):
         _id = self._id_of_key(k)
         try:
             data = self.store[_id]
-        except self._errors_that_trigger_missing:
-            data = self.__missing__(k)
+        except self._errors_that_trigger_missing as error:
+            if hasattr(self, "__missing__"):
+                data = self.__missing__(k)
+            else:
+                raise error
         return self._obj_of_data(data)
 
-    def __missing__(self, k):
-        raise KeyError(k)
-
     def get(self, k: Key, default=None) -> Val:
         try:
             return self[k]
@@ -645,7 +663,6 @@ class Store(KvPersister):
             for k in self:
                 return k, self[k]
         except Exception as e:
-
             from warnings import warn
 
             if k is None:
@@ -656,13 +673,13 @@ class Store(KvPersister):
                 msg += (
                     "... because there's a layer transforming outcoming keys that are not the ones the store actually "
                     "uses? If you didn't wrap the store with the inverse ingoing keys transformation, "
-                    'that would happen.\n'
+                    "that would happen.\n"
                 )
                 msg += (
                     "I'll ask the inner-layer what it's head is, but IT MAY NOT REFLECT the reality of your store "
-                    'if you have some filtering, caching etc.'
+                    "if you have some filtering, caching etc."
                 )
-                msg += f'The error messages was: \n{e}'
+                msg += f"The error messages was: \n{e}"
                 warn(msg)
 
             for _id in self.store:
@@ -699,7 +716,7 @@ class Store(KvPersister):
         if isinstance(self._max_repr_size, int):
             half = int(self._max_repr_size)
             if len(x) > self._max_repr_size:
-                x = x[:half] + '  ...  ' + x[-half:]
+                x = x[:half] + "  ...  " + x[-half:]
         return x
         # return self.store.__repr__()
 
@@ -722,10 +739,12 @@ KvStore = Store  # alias with explict name
 ########################################################################################################################
 # walking in trees
 
-from typing import Callable, KT, VT, Any, TypeVar, Iterator
+from typing import KT, VT, Any, TypeVar
+from collections.abc import Callable, Iterator
+from collections import deque
 
-PT = TypeVar('PT')  # Path Type
-inf = float('infinity')
+PT = TypeVar("PT")  # Path Type
+inf = float("infinity")
 
 
 def val_is_mapping(p: PT, k: KT, v: VT) -> bool:
@@ -736,7 +755,7 @@ def asis(p: PT, k: KT, v: VT) -> Any:
     return p, k, v
 
 
-def tuple_keypath_and_val(p: PT, k: KT, v: VT) -> Tuple[PT, VT]:
+def tuple_keypath_and_val(p: PT, k: KT, v: VT) -> tuple[PT, VT]:
     if p == ():  # we're just begining (the root),
         p = (k,)  # so begin the path with the first key.
     else:
@@ -748,21 +767,30 @@ def tuple_keypath_and_val(p: PT, k: KT, v: VT) -> Tuple[PT, VT]:
 #  This one even merits an extensive usage and example tutorial!
 def kv_walk(
     v: Mapping,
-    yield_func: Callable[[PT, KT, VT], Any] = asis,
+    leaf_yield: Callable[[PT, KT, VT], Any] = asis,
     walk_filt: Callable[[PT, KT, VT], bool] = val_is_mapping,
-    pkv_to_pv: Callable[[PT, KT, VT], Tuple[PT, VT]] = tuple_keypath_and_val,
+    pkv_to_pv: Callable[[PT, KT, VT], tuple[PT, VT]] = tuple_keypath_and_val,
+    *,
+    branch_yield: Callable[[PT, KT, VT], Any] = None,
+    breadth_first: bool = False,
     p: PT = (),
 ) -> Iterator[Any]:
     """
     Walks a nested structure of mappings, yielding stuff on the way.
 
     :param v: A nested structure of mappings
-    :param yield_func: (pp, k, vv) -> what ever you want the gen to yield
+    :param leaf_yield: (pp, k, vv) -> Any, what you want to yield when you encounter
+        a leaf node (as define by walk_filt resolving to False)
     :param walk_filt: (p, k, vv) -> (bool) whether to explore the nested structure v further
     :param pkv_to_pv:  (p, k, v) -> (pp, vv)
         where pp is a form of p + k (update of the path with the new node k)
-        and vv is the value that will be used by both walk_filt and yield_func
+        and vv is the value that will be used by both walk_filt and leaf_yield
     :param p: The path to v (used internally, mainly, to keep track of the path)
+    :param breadth_first: Whether to perform breadth-first traversal
+        (instead of the default depth-first traversal).
+    :param branch_yield: (pp, k, vv) -> Any, optional yield function to yield before
+        the recursive walk of a branch. This is useful if you want to yield something
+        for every branch, not just the leaves.
 
     >>> d = {'a': 1, 'b': {'c': 2, 'd': 3}}
     >>> list(kv_walk(d))
@@ -784,7 +812,7 @@ def kv_walk(
     >>> def leveled_map_walk(m, levels):
     ...     yield from kv_walk(
     ...         m,
-    ...         yield_func=lambda p, k, v: (p, v),
+    ...         leaf_yield=lambda p, k, v: (p, v),
     ...         walk_filt=mk_level_walk_filt(levels)
     ...     )
     >>> m = {
@@ -816,7 +844,7 @@ def kv_walk(
     ... )
 
     Tip: If you want to use ``kv_filt`` to search and extract stuff from a nested
-    mapping, you can have your ``yield_func`` return a sentinel (say, ``None``) to
+    mapping, you can have your ``leaf_yield`` return a sentinel (say, ``None``) to
     indicate that the value should be skipped, and then filter out the ``None``s from
     your results.
 
@@ -826,31 +854,86 @@ def kv_walk(
     ...     'aaa': {'bbb': 314},
     ... }
     >>> return_path_if_int_leaf = lambda p, k, v: (p, v) if isinstance(v, int) else None
-    >>> list(filter(None, kv_walk(mm, yield_func=return_path_if_int_leaf)))
+    >>> list(filter(None, kv_walk(mm, leaf_yield=return_path_if_int_leaf)))
     [(('a', 'b', 'c'), 42), (('aaa', 'bbb'), 314)]
 
     This "path search" functionality is available as a function in the ``recipes``
     module, as ``search_paths``.
 
-    Inspiration: ``kv_walk`` was inspired by `remap` from the `boltons` package.
-    You may consider using that instead, as it has a much more extensive documetation:
-    See https://sedimental.org/remap.html for example.
+    One last thing. Let's demonstrate the use of `branch_yield` and `breadth_first`.
+    Consider the following dictionary:
+
+    >>> d = {'big': {'apple': 1}, 'deal': 3, 'apple': {'pie': 1, 'crumble': 2}}
+
+    Say you wanted to find all the paths that end with 'apple'. You could do:
+
+    >>> from functools import partial
+    >>> yield_path_if_ends_with_apple = lambda p, k, v: p if k == 'apple' else None
+    >>> walker1 = partial(kv_walk, leaf_yield=yield_path_if_ends_with_apple)
+    >>> list(filter(None, walker1(d)))
+    [('big', 'apple')]
+
+    It only got `('big', 'apple')` because the `leaf_yield` is only triggered
+    for leaf nodes (as defined by the `walk_filt` argument, which defaults to
+    `val_is_mapping`). So let's try again, but this time, we'll use `branch_yield`
+    to yield the path for every branch (not just the leaves):
+
+    >>> walker2 = partial(walker1, branch_yield=yield_path_if_ends_with_apple)
+    >>> list(filter(None, walker2(d)))
+    [('big', 'apple'), ('apple',)]
+
+    But this isn't convenient if you'd like your search to finish as soon as you
+    find a path ending with `'apple'`. The order here comes from the fact that
+    `kv_walk` does a depth-first traversal. If you want to do a breadth-first
+    traversal, just say it:
+
+    >>> walker3 = partial(walker2, breadth_first=True)
+    >>> list(filter(None, walker3(d)))
+    [('apple',), ('big', 'apple')]
+
+    So now, you can get the first apple path by doing:
+    >>> next(filter(None, walker3(d)))
+    ('apple',)
 
     """
-    # print(f"1: entered with: v={v}, p={p}")
-    for k, vv in v.items():
-        # print(f"2: item: k={k}, vv={vv}")
-        pp, vv = pkv_to_pv(
-            p, k, vv
-        )  # update the path with k (and preprocess v if necessary)
-        if walk_filt(
-            p, k, vv
-        ):  # should we recurse? (based on some function of p, k, v)
-            # print(f"3: recurse with: pp={pp}, vv={vv}\n")
-            yield from kv_walk(vv, yield_func, walk_filt, pkv_to_pv, pp)  # recurse
-        else:
-            # print(f"4: yield_func(pp={pp}, k={k}, vv={vv})\n --> {yield_func(pp, k, vv)}")
-            yield yield_func(pp, k, vv)  # yield something computed from p, k, vv
+    if not breadth_first:
+        # print(f"1: entered with: v={v}, p={p}")
+        for k, vv in v.items():
+            # print(f"2: item: k={k}, vv={vv}")
+            pp, vv = pkv_to_pv(
+                p, k, vv
+            )  # update the path with k (and preprocess v if necessary)
+            if walk_filt(
+                p, k, vv
+            ):  # should we recurse? (based on some function of p, k, v)
+                # print(f"3: recurse with: pp={pp}, vv={vv}\n")
+                if branch_yield:
+                    yield branch_yield(pp, k, vv)
+                yield from kv_walk(
+                    vv,
+                    leaf_yield,
+                    walk_filt,
+                    pkv_to_pv,
+                    breadth_first=breadth_first,
+                    branch_yield=branch_yield,
+                    p=pp,
+                )  # recurse
+            else:
+                # print(f"4: leaf_yield(pp={pp}, k={k}, vv={vv})\n --> {leaf_yield(pp, k, vv)}")
+                yield leaf_yield(pp, k, vv)  # yield something computed from p, k, vv
+    else:
+        queue = deque([(p, v)])
+
+        while queue:
+            p, v = queue.popleft()
+            for k, vv in v.items():
+                pp, vv = pkv_to_pv(p, k, vv)
+                if walk_filt(p, k, vv):
+                    if branch_yield:
+                        yield branch_yield(pp, k, vv)
+                    queue.append((pp, vv))
+                else:
+                    yield leaf_yield(pp, k, vv)
 
 
 def has_kv_store_interface(o):
@@ -863,10 +946,10 @@ def has_kv_store_interface(o):
 
     """
     return (
-        hasattr(o, '_id_of_key')
-        and hasattr(o, '_key_of_id')
-        and hasattr(o, '_data_of_obj')
-        and hasattr(o, '_obj_of_data')
+        hasattr(o, "_id_of_key")
+        and hasattr(o, "_key_of_id")
+        and hasattr(o, "_data_of_obj")
+        and hasattr(o, "_obj_of_data")
     )
 
 
@@ -907,12 +990,12 @@ class KeyValidationABC(metaclass=ABCMeta):
 
     def check_key_is_valid(self, k):
         if not self.is_valid_key(k):
-            raise KeyValidationError('key is not valid: {}'.format(k))
+            raise KeyValidationError(f"key is not valid: {k}")
 
     @classmethod
     def __subclasshook__(cls, C):
         if cls is KeyValidationABC:
-            return _check_methods(C, 'is_valid_key', 'check_key_is_valid')
+            return _check_methods(C, "is_valid_key", "check_key_is_valid")
         return NotImplemented
 
 
@@ -1011,7 +1094,7 @@ class Stream:
     def __init__(self, stream):
         self.stream = stream
 
-    wrap = classmethod(partial(delegator_wrap, delegation_attr='stream'))
+    wrap = classmethod(partial(delegator_wrap, delegation_attr="stream"))
 
     # _data_of_obj = static_identity_method  # for write methods
     _pre_iter = static_identity_method
