@@ -101,13 +101,15 @@ def items_with_caught_exceptions(
         try:
             v = d[k]  # try getting the value...
             yield k, v  # and if you do, yield the (k, v) pair
-        except catch_exceptions as e:  # catch the specific exceptions you requested to catch
+        except (
+            catch_exceptions
+        ) as e:  # catch the specific exceptions you requested to catch
             t = wrapped_callback(k=k, e=e, d=d, i=i)  # call it
             if yield_callback_output:  # if the user wants the output of the callback
                 yield t  # yield it
 
 
-def _assert_condition(condition, err_msg='', err_cls=AssertionError):
+def _assert_condition(condition, err_msg="", err_cls=AssertionError):
     if not condition:
         raise err_cls(err_msg)
 
