@@ -1,7 +1,5 @@
-"""Recipes using dol
+"""Recipes using dol"""
 
-
-"""
-__all__ = ['search_paths']
+__all__ = ["search_paths"]
 
 from dol.paths import path_filter as search_paths  # was recipe. Promoted to paths
