@@ -1,7 +1,9 @@
 """
 This module contains key-value views of disparate sources.
 """
-from typing import Iterator, Mapping, Iterable, Callable, Union, Any
+
+from typing import Union, Any
+from collections.abc import Iterator, Mapping, Iterable, Callable
 from operator import itemgetter
 from itertools import groupby as itertools_groupby
 
@@ -39,13 +41,13 @@ class NotUnique(ValueError):
     """Raised when an iterator was expected to have only one element, but had more"""
 
 
-NoMoreElements = type('NoMoreElements', (object,), {})()
+NoMoreElements = type("NoMoreElements", (object,), {})()
 
 
 def unique_element(iterator):
     element = next(iterator)
     if next(iterator, NoMoreElements) is not NoMoreElements:
-        raise NotUnique('iterator had more than one element')
+        raise NotUnique("iterator had more than one element")
     return element
 
 
@@ -77,6 +79,7 @@ def _kv_spec_to_func(kv_spec: KvSpec) -> Callable:
 class FlatReader(KvReader):
     """Get a 'flat view' of a store of stores.
     That is, where keys are `(first_level_key, second_level_key)` pairs.
+    This is useful, for instance, to make a union of stores (you'll get all the values).
 
     >>> readers = {
     ...     'fr': {1: 'un', 2: 'deux'},
@@ -109,7 +112,8 @@ from collections import ChainMap
 
 
 from collections import ChainMap
-from typing import Callable, Iterable, Iterator, Mapping, TypedDict, Union
+from typing import TypedDict, Union
+from collections.abc import Callable, Iterable, Iterator, Mapping
 
 from dol.base import KvPersister, KvReader
 from dol.trans import wrap_kvs
@@ -145,14 +149,14 @@ class FanoutReader(KvReader):
     >>> reader['b']
     {'bytes_store': b'b', 'metadata_store': {'x': 2}}
 
-    The reader returns a dict with the values from each store, keyed by the name of the 
+    The reader returns a dict with the values from each store, keyed by the name of the
     store.
 
     We can also pass a default value to return if the key is not in the store:
 
     >>> reader = FanoutReader(
     ...     stores=stores,
-    ...     default='no value in this store for this key', 
+    ...     default='no value in this store for this key',
     ... )
     >>> reader['a']
     {'bytes_store': b'a', 'metadata_store': 'no value in this store for this key'}
@@ -179,8 +183,16 @@ class FanoutReader(KvReader):
         self,
         stores: Mapping[Any, Mapping],
         default: Any = None,
+        *,
         get_existing_values_only: bool = False,
     ):
+        if not isinstance(stores, Mapping):
+            if isinstance(stores, Iterable):
+                stores = dict(enumerate(stores))
+            else:
+                raise ValueError(
+                    f"stores must be a Mapping or an Iterable, not {type(stores)}"
+                )
         self._stores = stores
         self._default = default
         self._get_existing_values_only = get_existing_values_only
@@ -190,7 +202,7 @@ class FanoutReader(KvReader):
         """A way to create a fan-out store from a mix of args and kwargs, instead of a
         single dict.
 
-        param args: sub-stores used to fan-out the data. These stores will be 
+        param args: sub-stores used to fan-out the data. These stores will be
             represented by their index in the tuple.
         param kwargs: sub-stores used to fan-out the data. These stores will be
             represented by their name in the dict. __init__ arguments can also be passed
@@ -216,7 +228,7 @@ class FanoutReader(KvReader):
         >>> reader['b']
         {0: b'b', 1: {'x': 2}}
 
-        The reader returns a dict with the values from each store, keyed by the index of 
+        The reader returns a dict with the values from each store, keyed by the index of
         the store in the `args` tuple.
 
         We can also create a fan-out reader passing the stores in kwargs:
@@ -236,8 +248,8 @@ class FanoutReader(KvReader):
         >>> reader['b']
         {0: b'b', 'metadata_store': {'x': 2}}
 
-        Note that the order of the stores is determined by the order of the args and 
-        kwargs.     
+        Note that the order of the stores is determined by the order of the args and
+        kwargs.
         """
 
         def extract_init_kwargs():
@@ -255,15 +267,19 @@ class FanoutReader(KvReader):
         return ChainMap(*self._stores.values())
 
     def __getitem__(self, k):
-        v = {
+        value_for_key_for_every_store = {
             store_key: store.get(k, self._default)
             for store_key, store in self._stores.items()
         }
-        if all(v == self._default for v in v.values()):
+        if all(v is self._default for v in value_for_key_for_every_store.values()):
             raise KeyError(k)
         if self._get_existing_values_only:
-            v = {k: v for k, v in v.items() if v != self._default}
-        return v
+            value_for_key_for_every_store = {
+                k: v
+                for k, v in value_for_key_for_every_store.items()
+                if v != self._default
+            }
+        return value_for_key_for_every_store
 
     def __iter__(self) -> Iterator:
         return iter(self._keys)
@@ -287,7 +303,7 @@ class FanoutPersister(FanoutReader, KvPersister):
         If False, only the stores that are set will be updated.
     param ignore_non_existing_store_keys: If True, ignore store keys from the value that
         are not in the persister. If False, a ValueError is raised.
-    
+
     Let's create a persister from in-memory stores:
 
     >>> bytes_store = dict()
@@ -324,8 +340,8 @@ class FanoutPersister(FanoutReader, KvPersister):
     Traceback (most recent call last):
         ...
     ValueError: All stores must be set when setting a value. Missing stores: {'metadata_store'}
-    
-    By default, if a store key from the value is not in the persister, a ValueError is 
+
+    By default, if a store key from the value is not in the persister, a ValueError is
     raised:
 
     >>> persister['a'] = dict(
@@ -370,7 +386,7 @@ class FanoutPersister(FanoutReader, KvPersister):
         ...
     KeyError: 'z'
 
-    However, if the key is in some of the stores, but not in others, the key is deleted 
+    However, if the key is in some of the stores, but not in others, the key is deleted
     from the stores where it is present:
 
     >>> bytes_store=dict(a=b'a')
@@ -388,6 +404,7 @@ class FanoutPersister(FanoutReader, KvPersister):
         self,
         stores: Mapping[Any, Mapping],
         default: Any = None,
+        *,
         get_existing_values_only: bool = False,
         need_to_set_all_stores: bool = False,
         ignore_non_existing_store_keys: bool = False,
@@ -405,14 +422,14 @@ class FanoutPersister(FanoutReader, KvPersister):
         if self._need_to_set_all_stores and not set(self._stores).issubset(set(v)):
             missing_stores = set(self._stores) - set(v)
             raise ValueError(
-                f'All stores must be set when setting a value. Missing stores: {missing_stores}'
+                f"All stores must be set when setting a value. Missing stores: {missing_stores}"
             )
         if not self._ignore_non_existing_store_keys and not set(v).issubset(
             set(self._stores)
         ):
             invalid_store_keys = set(v) - set(self._stores)
             raise ValueError(
-                f'The value contains some invalid store keys: {invalid_store_keys}'
+                f"The value contains some invalid store keys: {invalid_store_keys}"
             )
         for store_key, vv in v.items():
             if store_key in self._stores:
@@ -428,6 +445,103 @@ class FanoutPersister(FanoutReader, KvPersister):
             del store[k]
 
 
+NotFound = type("NotFound", (object,), {})()
+
+
+@wrap_kvs(value_encoder=lambda self, v: {k: v for k in self._stores.keys()})
+class CascadedStores(FanoutPersister):
+    """
+    A MutableMapping interface to a collection of stores that will write a value in
+    all the stores it contains, read it from the first store it finds that has it, and
+    write it back to all the stores up to the store where it found it.
+
+    This is useful, for example, when you want to, say, write something to disk,
+    and possibly to a remote backup or shared store, but also keep that value in memory.
+
+    The name `CascadedStores` comes from "Cascaded Caches", which is a common pattern in
+    caching systems
+    (e.g. https://philipwalton.com/articles/cascading-cache-invalidation/)
+
+    To demo this, let's create a couple of stores that print when they get a value:
+
+
+    >>> from collections import UserDict
+    >>> class LoggedDict(UserDict):
+    ...     def __init__(self, name: str):
+    ...        self.name = name
+    ...        super().__init__()
+    ...     def __getitem__(self, k):
+    ...         print(f"Getting {k} from {self.name}")
+    ...         return super().__getitem__(k)
+    >>> cache = LoggedDict('cache')
+    >>> disk = LoggedDict('disk')
+    >>> remote = LoggedDict('remote')
+
+    Now we can create a CascadedStores instance with these stores and write a
+    value to it:
+
+    >>> stores = CascadedStores([cache, disk, remote])
+    >>> stores['f'] = 42
+
+    See that it's in both stores:
+
+    >>> cache['f']
+    Getting f from cache
+    42
+    >>> disk['f']
+    Getting f from disk
+    42
+    >>> remote['f']
+    Getting f from remote
+    42
+
+    See how it reads from the first store only, because it found the `f` key there:
+
+    >>> stores['f']
+    Getting f from cache
+    42
+
+    Let's write something in disk only:
+
+    >>> disk['g'] = 43
+
+    Now if you ask for `g`, it won't find it in cache, but will find it in `disk`
+    and return it.
+
+    >>> stores['g']
+    Getting g from disk
+    43
+
+    Here's the thing though. Now, `g` is also in `cache`:
+
+    >>> cache
+    {'f': 42, 'g': 43}
+
+    But `remote` still only has `f`:
+
+    >>> remote
+    {'f': 42}
+
+
+    """
+
+    # Note: Need to overwrite FanoutPersister's getitem to not read values from all stores
+    def __getitem__(self, k):
+        """Returns the value of the first store for that key"""
+        for store_ref, store in self._stores.items():
+            if k in store:  # Check existence first, without triggering __getitem__
+                v = store[k]  # Now get the value (will trigger logging)
+                # value found, now let's write it to all the stores up to the store_ref
+                for _store_ref, _store in self._stores.items():
+                    if _store_ref != store_ref:
+                        _store[k] = v
+                    else:
+                        break
+                # now return the value
+                return v
+        raise KeyError(k)
+
+
 class SequenceKvReader(KvReader):
     """
     A KvReader that sources itself in an iterable of elements from which keys and values
@@ -572,7 +686,7 @@ class SequenceKvReader(KvReader):
         for kk, vv in self.kv_items():
             if kk == k:
                 return vv
-        raise KeyError(f'Key not found: {k}')
+        raise KeyError(f"Key not found: {k}")
 
     def __iter__(self):
         yield from map(itemgetter(0), self.kv_items())
@@ -628,7 +742,7 @@ class FuncReader(KvReader):
 
     """
 
-    def __init__(self, funcs):
+    def __init__(self, funcs: Mapping[str, Callable] | Iterable[Callable]):
         # TODO: assert no free arguments (arguments are allowed but must all have
         #  defaults)
         if isinstance(funcs, Mapping):
@@ -663,45 +777,24 @@ import os
 
 psep = os.path.sep
 
-ddir = lambda o: [x for x in dir(o) if not x.startswith('_')]
+ddir = lambda o: [x for x in dir(o) if not x.startswith("_")]
 
 
 def not_underscore_prefixed(x):
-    return not x.startswith('_')
+    return not x.startswith("_")
 
 
 def _path_to_module_str(path, root_path):
-    assert path.endswith('.py')
+    assert path.endswith(".py")
     path = path[:-3]
     if root_path.endswith(psep):
         root_path = root_path[:-1]
     root_path = os.path.dirname(root_path)
     len_root = len(root_path) + 1
     path_parts = path[len_root:].split(psep)
-    if path_parts[-1] == '__init__.py':
+    if path_parts[-1] == "__init__.py":
         path_parts = path_parts[:-1]
-    return '.'.join(path_parts)
-
-
-class ObjReader(KvReader):
-    def __init__(self, obj):
-        self.src = obj
-        copy_attrs(
-            target=self,
-            source=self.src,
-            attrs=('__name__', '__qualname__', '__module__'),
-            raise_error_if_an_attr_is_missing=False,
-        )
-
-    def __repr__(self):
-        return f'{self.__class__.__qualname__}({self.src})'
-
-    @property
-    def _source(self):
-        from warnings import warn
-
-        warn('Deprecated: Use .src instead of ._source', DeprecationWarning, 2)
-        return self.src
+    return ".".join(path_parts)
 
 
 # class SourceReader(KvReader):
@@ -712,9 +805,76 @@ class ObjReader(KvReader):
 #     def __init__(self, obj, src_to_key, key_filt=None, ):
 
 
+class ObjLoader:
+    def __init__(self, data_of_key, obj_of_data=None):
+        self.data_of_key = data_of_key
+        if obj_of_data is not None or not callable(obj_of_data):
+            raise TypeError("serializer must be None or a callable")
+        self.obj_of_data = obj_of_data
+
+    def __call__(self, k):
+        if self.obj_of_data is not None:
+            return self.obj_of_data(self.data_of_key(k))
+        else:
+            return self.data_of_key(k)
+
+
+# TODO: See explicit.py module and FuncReader above for near duplicates!
+# TODO: Add an obj_of_key argument to wrap_kvs? (Or should it be data_of_key?)
+# TODO: Another near-duplicate found: dol.paths.PathMappedData
+# Note: Older version commmented below
+class ObjReader:
+    """
+    A reader that uses a specified function to get the contents for a given key.
+
+    >>> # define a contents_of_key that reads stuff from a dict
+    >>> data = {'foo': 'bar', 42: "everything"}
+    >>> def read_dict(k):
+    ...     return data[k]
+    >>> pr = ObjReader(_obj_of_key=read_dict)
+    >>> pr['foo']
+    'bar'
+    >>> pr[42]
+    'everything'
+    >>>
+    >>> # define contents_of_key that reads stuff from a file given it's path
+    >>> def read_file(path):
+    ...     with open(path) as fp:
+    ...         return fp.read()
+    >>> pr = ObjReader(_obj_of_key=read_file)
+    >>> file_where_this_code_is = __file__
+
+    ``file_where_this_code_is`` should be the file where this doctest is written,
+    therefore should contain what I just said:
+
+    >>> 'therefore should contain what I just said' in pr[file_where_this_code_is]
+    True
+
+    """
+
+    def __init__(self, _obj_of_key: Callable):
+        self._obj_of_key = _obj_of_key
+
+    @classmethod
+    def from_composition(cls, data_of_key, obj_of_data=None):
+        return cls(
+            _obj_of_key=ObjLoader(data_of_key=data_of_key, obj_of_data=obj_of_data)
+        )
+
+    def __getitem__(self, k):
+        try:
+            return self._obj_of_key(k)
+        except Exception as e:
+            raise KeyError(
+                "KeyError in {} when trying to __getitem__({}): {}".format(
+                    e.__class__.__name__, k, e
+                )
+            )
+
+
 # Pattern: Recursive navigation
 # Note: Moved dev to independent package called "guide"
-@cached_keys(keys_cache=set, name='Attrs')
+@cached_keys(keys_cache=set, name="Attrs")
 class Attrs(ObjReader):
     """A simple recursive KvReader for the attributes of a python object.
     Keys are attr names, values are Attrs(attr_val) instances.
@@ -742,7 +902,7 @@ class Attrs(ObjReader):
                 try:
                     name = _path_to_module_str(path, root_path)
                 except Exception:
-                    name = 'fake.module.name'
+                    name = "fake.module.name"
         spec = importlib.util.spec_from_file_location(name, path)
         foo = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(foo)
@@ -755,7 +915,7 @@ class Attrs(ObjReader):
         return self.__class__(getattr(self.src, k), self._key_filt, self.getattrs)
 
     def __repr__(self):
-        return f'{self.__class__.__qualname__}({self.src}, {self._key_filt})'
+        return f"{self.__class__.__qualname__}({self.src}, {self._key_filt})"
 
 
 Ddir = Attrs  # for back-compatibility, temporarily
@@ -764,17 +924,17 @@ import re
 
 
 def _extract_first_identifier(string: str) -> str:
-    m = re.match(r'\w+', string)
+    m = re.match(r"\w+", string)
     if m:
         return m.group(0)
     else:
-        return ''
+        return ""
 
 
-def _dflt_object_namer(obj, dflt_name: str = 'name_not_found'):
+def _dflt_object_namer(obj, dflt_name: str = "name_not_found"):
     return (
-        getattr(obj, '__name__', None)
-        or _extract_first_identifier(getattr(obj, '__doc__'))
+        getattr(obj, "__name__", None)
+        or _extract_first_identifier(getattr(obj, "__doc__"))
         or dflt_name
     )
 
@@ -840,7 +1000,7 @@ class AttrContainer:
             self._validate_named_objects(auto_named_objects, named_objects)
             named_objects = dict(auto_named_objects, **named_objects)
 
-        super().__setattr__('_source', {})
+        super().__setattr__("_source", {})
         for k, v in named_objects.items():
             setattr(self, k, v)
 
@@ -848,13 +1008,13 @@ class AttrContainer:
     def _validate_named_objects(auto_named_objects, named_objects):
         if not all(map(str.isidentifier, auto_named_objects)):
             raise ValueError(
-                'All names produced by _object_namer should be valid python identifiers:'
+                "All names produced by _object_namer should be valid python identifiers:"
                 f" {', '.join(x for x in auto_named_objects if not x.isidentifier())}"
             )
         clashing_names = auto_named_objects.keys() & named_objects.keys()
         if clashing_names:
             raise ValueError(
-                'Some auto named objects clashed with named ones: '
+                "Some auto named objects clashed with named ones: "
                 f"{', '.join(clashing_names)}"
             )
 
@@ -928,3 +1088,24 @@ class AttrDict(AttrContainer, KvPersister):
 
     .. seealso:: Objects in ``py2store.utils.attr_dict`` module
     """
+
+
+# class ObjReader(KvReader):
+#     def __init__(self, obj):
+#         self.src = obj
+#         copy_attrs(
+#             target=self,
+#             source=self.src,
+#             attrs=('__name__', '__qualname__', '__module__'),
+#             raise_error_if_an_attr_is_missing=False,
+#         )
+
+#     def __repr__(self):
+#         return f'{self.__class__.__qualname__}({self.src})'
+
+#     @property
+#     def _source(self):
+#         from warnings import warn
+
+#         warn('Deprecated: Use .src instead of ._source', DeprecationWarning, 2)
+#         return self.src
