@@ -1,7 +1,7 @@
 from dol import Store
 import pickle
 
-s = Store({'a': 1, 'b': 2})
+s = Store({"a": 1, "b": 2})
 t = pickle.dumps(s)
 ss = pickle.loads(t)
 
