@@ -1,11 +1,13 @@
 """
 Data object layers and other utils to work with zip files.
 """
+
 import os
 from pathlib import Path
 from io import BytesIO
 from functools import partial, wraps
-from typing import Callable, Union, Iterable
+from typing import Union, Literal
+from collections.abc import Callable, Iterable, Mapping
 import zipfile
 from zipfile import (
     ZipFile,
@@ -16,39 +18,41 @@ from zipfile import (
     ZIP_LZMA,
 )
 from dol.base import KvReader, KvPersister
+from dol.trans import filt_iter
 from dol.filesys import FileCollection, Files
 from dol.util import lazyprop, fullpath
 from dol.sources import FlatReader
 
 __all__ = [
-    'COMPRESSION',
-    'DFLT_COMPRESSION',
-    'compression_methods',
-    'to_zipped_bytes',
-    'zipped_bytes_to_bytes',
-    'to_zip_file',
-    'file_or_folder_to_zip_file',
-    'if_i_zipped_stats',
-    'ZipReader',
-    'ZipInfoReader',
-    'ZipFilesReader',
-    'ZipFilesReaderAndBytesWriter',
-    'FlatZipFilesReader',
-    'mk_flatzips_store',
-    'FilesOfZip',
-    'FileStreamsOfZip',
-    'ZipFileStreamsReader',
-    'OverwriteNotAllowed',
-    'EmptyZipError',
-    'ZipStore',
-    'remove_some_entries_from_zip',
-    'remove_mac_junk_from_zip',
+    "COMPRESSION",
+    "DFLT_COMPRESSION",
+    "compression_methods",
+    "zip_compress",
+    "zip_decompress",
+    "to_zip_file",
+    "file_or_folder_to_zip_file",
+    "if_i_zipped_stats",
+    "ZipReader",
+    "ZipInfoReader",
+    "ZipFilesReader",
+    "ZipFilesReaderAndBytesWriter",
+    "FlatZipFilesReader",
+    "mk_flatzips_store",
+    "FilesOfZip",
+    "FileStreamsOfZip",
+    "ZipFileStreamsReader",
+    "OverwriteNotAllowed",
+    "EmptyZipError",
+    "ZipStore",
+    "ZipFiles",
+    "remove_some_entries_from_zip",
+    "remove_mac_junk_from_zip",
 ]
 
 # TODO: Do all systems have this? If not, need to choose dflt carefully
 #  (choose dynamically?)
 DFLT_COMPRESSION = zipfile.ZIP_DEFLATED
-DFLT_ENCODING = 'utf-8'
+DFLT_ENCODING = "utf-8"
 
 
 class COMPRESSION:
@@ -63,10 +67,10 @@ class COMPRESSION:
 
 
 compression_methods = {
-    'stored': zipfile.ZIP_STORED,  # doesn't even compress
-    'deflated': zipfile.ZIP_DEFLATED,  # usual zip compression method
-    'bzip2': zipfile.ZIP_BZIP2,  # BZIP2 compression method.
-    'lzma': zipfile.ZIP_LZMA,  # LZMA compression method
+    "stored": zipfile.ZIP_STORED,  # doesn't even compress
+    "deflated": zipfile.ZIP_DEFLATED,  # usual zip compression method
+    "bzip2": zipfile.ZIP_BZIP2,  # BZIP2 compression method.
+    "lzma": zipfile.ZIP_LZMA,  # LZMA compression method
 }
 
 
@@ -74,9 +78,9 @@ def take_everything(fileinfo):
     return True
 
 
-def to_zipped_bytes(
-    b: Union[bytes, str],
-    filename='some_bytes',
+def zip_compress(
+    b: bytes | str,
+    filename="some_bytes",
     *,
     compression=DFLT_COMPRESSION,
     allowZip64=True,
@@ -90,21 +94,21 @@ def to_zipped_bytes(
     >>> len(b)
     2000
     >>>
-    >>> zipped_bytes = to_zipped_bytes(b)
+    >>> zipped_bytes = zip_compress(b)
     >>> # Note: Compression details will be system dependent
     >>> len(zipped_bytes)  # doctest: +SKIP
     137
-    >>> unzipped_bytes = zipped_bytes_to_bytes(zipped_bytes)
+    >>> unzipped_bytes = zip_decompress(zipped_bytes)
     >>> unzipped_bytes == b  # verify that unzipped bytes are the same as the original
     True
     >>>
     >>> from dol.zipfiledol import compression_methods
     >>>
-    >>> zipped_bytes = to_zipped_bytes(b, compression=compression_methods['bzip2'])
+    >>> zipped_bytes = zip_compress(b, compression=compression_methods['bzip2'])
     >>> # Note: Compression details will be system dependent
     >>> len(zipped_bytes)  # doctest: +SKIP
     221
-    >>> unzipped_bytes = zipped_bytes_to_bytes(zipped_bytes)
+    >>> unzipped_bytes = zip_decompress(zipped_bytes)
     >>> unzipped_bytes == b  # verify that unzipped bytes are the same as the original
     True
     """
@@ -117,17 +121,21 @@ def to_zipped_bytes(
     bytes_buffer = BytesIO()
     if isinstance(b, str):  # if b is a string, need to convert to bytes
         b = b.encode(encoding)
-    with ZipFile(bytes_buffer, 'w', **kwargs) as fp:
+    with ZipFile(bytes_buffer, "w", **kwargs) as fp:
         fp.writestr(filename, b)
     return bytes_buffer.getvalue()
 
 
-def zipped_bytes_to_bytes(
-    b: bytes, *, allowZip64=True, compresslevel=None, strict_timestamps=True,
+def zip_decompress(
+    b: bytes,
+    *,
+    allowZip64=True,
+    compresslevel=None,
+    strict_timestamps=True,
 ) -> bytes:
     """Decompress input bytes of a single file zip, returning the uncompressed bytes
 
-    See ``to_zipped_bytes`` for usage examples.
+    See ``zip_compress`` for usage examples.
     """
     kwargs = dict(
         allowZip64=allowZip64,
@@ -135,26 +143,26 @@ def zipped_bytes_to_bytes(
         strict_timestamps=strict_timestamps,
     )
     bytes_buffer = BytesIO(b)
-    with ZipFile(bytes_buffer, 'r', **kwargs) as zip_file:
+    with ZipFile(bytes_buffer, "r", **kwargs) as zip_file:
         file_list = zip_file.namelist()
         if len(file_list) != 1:
-            raise RuntimeError('zipped_bytes_to_bytes only works with single file zips')
+            raise RuntimeError("zip_decompress only works with single file zips")
         filename = file_list[0]
-        with zip_file.open(filename, 'r') as fp:
+        with zip_file.open(filename, "r") as fp:
             file_bytes = fp.read()
     return file_bytes
 
 
 def _filename_from_zip_path(path):
     filename = path  # default
-    if path.endswith('.zip'):
+    if path.endswith(".zip"):
         filename, _ = os.path.splitext(os.path.basename(path))
     return filename
 
 
 # TODO: Look into pwd: Should we use it for setting pwd when pwd doesn't exist?
 def to_zip_file(
-    b: Union[bytes, str],
+    b: bytes | str,
     zip_filepath,
     filename=None,
     *,
@@ -171,7 +179,7 @@ def to_zip_file(
     :param encoding: In case the input is str, the encoding to use to convert to bytes
 
     """
-    z = ZipStore(
+    z = ZipFiles(
         zip_filepath,
         compression=compression,
         allow_overwrites=allow_overwrites,
@@ -195,9 +203,9 @@ def file_or_folder_to_zip_file(
     """Zip input bytes and save to a single-file zip file."""
 
     if zip_filepath is None:
-        zip_filepath = os.path.basename(src_path) + '.zip'
+        zip_filepath = os.path.basename(src_path) + ".zip"
 
-    z = ZipStore(
+    z = ZipFiles(
         zip_filepath,
         compression=compression,
         allow_overwrites=allow_overwrites,
@@ -212,7 +220,7 @@ def file_or_folder_to_zip_file(
         for k, v in src.items():
             z[k] = v
     else:
-        raise FileNotFoundError(f'{src_path}')
+        raise FileNotFoundError(f"{src_path}")
 
 
 def if_i_zipped_stats(b: bytes):
@@ -237,23 +245,23 @@ def if_i_zipped_stats(b: bytes):
     import time
 
     stats = dict()
-    stats['uncompressed'] = {'bytes': len(b), 'comp_time': 0, 'uncomp_time': 0}
+    stats["uncompressed"] = {"bytes": len(b), "comp_time": 0, "uncomp_time": 0}
     for name, compression in compression_methods.items():
-        if name != 'stored':
+        if name != "stored":
             try:
-                stats[name] = dict.fromkeys(stats['uncompressed'])
+                stats[name] = dict.fromkeys(stats["uncompressed"])
                 tic = time.time()
-                compressed = to_zipped_bytes(b, compression=compression)
+                compressed = zip_compress(b, compression=compression)
                 elapsed = time.time() - tic
-                stats[name]['bytes'] = len(compressed)
-                stats[name]['comp_time'] = elapsed
+                stats[name]["bytes"] = len(compressed)
+                stats[name]["comp_time"] = elapsed
                 tic = time.time()
-                uncompressed = zipped_bytes_to_bytes(compressed)
+                uncompressed = zip_decompress(compressed)
                 elapsed = time.time() - tic
-                assert (
-                    uncompressed == b
-                ), 'the uncompressed bytes were different than the original'
-                stats[name]['uncomp_time'] = elapsed
+                assert uncompressed == b, (
+                    "the uncompressed bytes were different than the original"
+                )
+                stats[name]["uncomp_time"] = elapsed
             except Exception:
                 raise
                 pass
@@ -341,7 +349,14 @@ class ZipReader(KvReader):
         #  'odir/app/data/audio/d/1574288084739961/m/Ctor.json']
     """
 
-    def __init__(self, zip_file, prefix='', open_kws=None, file_info_filt=None):
+    def __init__(
+        self,
+        zip_file,
+        prefix="",
+        *,
+        open_kws=None,
+        file_info_filt=None,
+    ):
         """
 
         Args:
@@ -369,7 +384,7 @@ class ZipReader(KvReader):
         self.zip_file = zip_file
 
     @classmethod
-    def for_files_only(cls, zip_file, prefix='', open_kws=None, file_info_filt=None):
+    def for_files_only(cls, zip_file, prefix="", open_kws=None, file_info_filt=None):
         if file_info_filt is None:
             file_info_filt = ZipReader.FILES_ONLY
         else:
@@ -416,15 +431,15 @@ class ZipReader(KvReader):
         return True
 
     def __repr__(self):
-        args_str = ', '.join(
+        args_str = ", ".join(
             (
                 f"'{self.zip_file.filename}'",
                 f"'{self.prefix}'",
-                f'{self.open_kws}',
-                f'{self.file_info_filt}',
+                f"{self.open_kws}",
+                f"{self.file_info_filt}",
             )
         )
-        return f'{self.__class__.__name__}({args_str})'
+        return f"{self.__class__.__name__}({args_str})"
 
     # TODO: Unaware of trans (filters, key trans, etc.)
     def get_info_reader(self):
@@ -442,7 +457,7 @@ class ZipInfoReader(ZipReader):
 
 
 class FilesOfZip(ZipReader):
-    def __init__(self, zip_file, prefix='', open_kws=None):
+    def __init__(self, zip_file, prefix="", open_kws=None):
         super().__init__(
             zip_file,
             prefix=prefix,
@@ -477,7 +492,7 @@ class ZipFilesReader(FileCollection, KvReader):
     def __init__(
         self,
         rootdir,
-        subpath=r'.+\.zip',
+        subpath=r".+\.zip",
         pattern_for_field=None,
         max_levels=0,
         zip_reader=ZipReader,
@@ -488,7 +503,11 @@ class ZipFilesReader(FileCollection, KvReader):
         self.zip_reader_kwargs = zip_reader_kwargs
         if self.zip_reader is ZipReader:
             self.zip_reader_kwargs = dict(
-                dict(prefix='', open_kws=None, file_info_filt=ZipReader.FILES_ONLY,),
+                dict(
+                    prefix="",
+                    open_kws=None,
+                    file_info_filt=ZipReader.FILES_ONLY,
+                ),
                 **self.zip_reader_kwargs,
             )
 
@@ -496,7 +515,7 @@ class ZipFilesReader(FileCollection, KvReader):
         try:
             return self.zip_reader(k, **self.zip_reader_kwargs)
         except FileNotFoundError as e:
-            raise KeyError(f'FileNotFoundError: {e}')
+            raise KeyError(f"FileNotFoundError: {e}")
 
 
 class ZipFilesReaderAndBytesWriter(ZipFilesReader):
@@ -505,7 +524,7 @@ class ZipFilesReaderAndBytesWriter(ZipFilesReader):
     """
 
     def __setitem__(self, k, v):
-        with open(k, 'wb') as fp:
+        with open(k, "wb") as fp:
             fp.write(v)
 
 
@@ -612,23 +631,22 @@ from dol.paths import mk_relative_path_store
 from dol.util import partialclass
 
 ZipFileStreamsReader = mk_relative_path_store(
-    partialclass(ZipFilesReader, zip_reader=FileStreamsOfZip), prefix_attr='rootdir',
+    partialclass(ZipFilesReader, zip_reader=FileStreamsOfZip),
+    prefix_attr="rootdir",
 )
-ZipFileStreamsReader.__name__ = 'ZipFileStreamsReader'
-ZipFileStreamsReader.__qualname__ = 'ZipFileStreamsReader'
+ZipFileStreamsReader.__name__ = "ZipFileStreamsReader"
+ZipFileStreamsReader.__qualname__ = "ZipFileStreamsReader"
 ZipFileStreamsReader.__doc__ = (
-    '''Like ZipFilesReader, but objects returned are file streams instead.'''
+    """Like ZipFilesReader, but objects returned are file streams instead."""
 )
 
 from dol.errors import OverWritesNotAllowedError
 
 
-class OverwriteNotAllowed(FileExistsError, OverWritesNotAllowedError):
-    ...
+class OverwriteNotAllowed(FileExistsError, OverWritesNotAllowedError): ...
 
 
-class EmptyZipError(KeyError, FileNotFoundError):
-    ...
+class EmptyZipError(KeyError, FileNotFoundError): ...
 
 
 class _EmptyZipReader(KvReader):
@@ -643,7 +661,7 @@ class _EmptyZipReader(KvReader):
 
     def __getitem__(self, k):
         raise EmptyZipError(
-            'The store is empty: ZipStore(zip_filepath={self.zip_filepath})'
+            "The store is empty: ZipFiles(zip_filepath={self.zip_filepath})"
         )
 
     def open(self, *args, **kwargs):
@@ -668,28 +686,28 @@ class _EmptyZipReader(KvReader):
         # return OpenedNotExistingFile()
 
 
-# TODO: Revise ZipReader and ZipFilesReader architecture and make ZipStore be a subclass of
+# TODO: Revise ZipReader and ZipFilesReader architecture and make ZipFiles be a subclass of
 #  Reader if poss
 # TODO: What if I just want to zip a (single) file. What does dol offer for that?
 # TODO: How about set_obj (in misc.py)? Make it recognize the .zip extension and subextension (
 #  e.g. .txt.zip) serialize
-class ZipStore(KvPersister):
+class ZipFiles(KvPersister):
     """Zip read and writing.
     When you want to read zips, there's the `FilesOfZip`, `ZipReader`, or `ZipFilesReader` we
     know and love.
 
-    Sometimes though, you want to write to zips too. For this, we have `ZipStore`.
+    Sometimes though, you want to write to zips too. For this, we have `ZipFiles`.
 
-    Since ZipStore can write to a zip, it's read functionality is not going to assume static data,
+    Since ZipFiles can write to a zip, it's read functionality is not going to assume static data,
     and cache things, as your favorite zip readers did.
     This, and the acrobatics need to disguise the weird zipfile into something more... key-value
     natural,
     makes for a not so efficient store, out of the box.
 
     I advise using one of the zip readers if all you need to do is read, or subclassing or
-     wrapping ZipStore with caching layers if it is appropriate to you.
+     wrapping ZipFiles with caching layers if it is appropriate to you.
 
-    Let's verify that a ZipStore can indeed write data. First, we'll set things up!
+    Let's verify that a ZipFiles can indeed write data. First, we'll set things up!
 
     >>> from tempfile import gettempdir
     >>> import os
@@ -704,7 +722,7 @@ class ZipStore(KvPersister):
 
     Okay, test_zipfile doesn't exist (but will soon...)
 
-    >>> z = ZipStore(test_zipfile)
+    >>> z = ZipFiles(test_zipfile)
 
     See that the file still doesn't exist (it will only be created when we start writing)
 
@@ -758,7 +776,7 @@ class ZipStore(KvPersister):
     @property
     def zip_reader(self):
         if os.path.isfile(self.zip_filepath):
-            return ZipFile(self.zip_filepath, mode='r', **self._zipfile_init_kw)
+            return ZipFile(self.zip_filepath, mode="r", **self._zipfile_init_kw)
         else:
             return _EmptyZipReader(self.zip_filepath)
 
@@ -769,18 +787,21 @@ class ZipStore(KvPersister):
         )
 
     def __getitem__(self, k):
-        with self.zip_reader.open(k, **dict(self._open_kw, mode='r')) as fp:
+        with self.zip_reader.open(k, **dict(self._open_kw, mode="r")) as fp:
             return fp.read()
 
     def __repr__(self):
-        args_str = ', '.join(
-            (f"'{self.zip_filepath}'", f"'allow_overwrites={self.allow_overwrites}'",)
+        args_str = ", ".join(
+            (
+                f"'{self.zip_filepath}'",
+                f"'allow_overwrites={self.allow_overwrites}'",
+            )
         )
-        return f'{self.__class__.__name__}({args_str})'
+        return f"{self.__class__.__name__}({args_str})"
 
     def __contains__(self, k):
         try:
-            with self.zip_reader.open(k, **dict(self._open_kw, mode='r')) as fp:
+            with self.zip_reader.open(k, **dict(self._open_kw, mode="r")) as fp:
                 pass
             return True
         except (
@@ -813,7 +834,7 @@ class ZipStore(KvPersister):
                 if self.zip_writer_opened:
                     raise OverwriteNotAllowed(
                         f"When using the context mode, you're not allowed to overwrite an "
-                        f'existing key: {k}'
+                        f"existing key: {k}"
                     )
                 else:
                     raise OverwriteNotAllowed(
@@ -821,25 +842,25 @@ class ZipStore(KvPersister):
                     )
 
         if self.zip_writer_opened:
-            with self.zip_writer.open(k, **dict(self._open_kw, mode='w')) as fp:
+            with self.zip_writer.open(k, **dict(self._open_kw, mode="w")) as fp:
                 return fp.write(v)
         else:
             with ZipFile(
-                self.zip_filepath, mode='a', **self._zipfile_init_kw
+                self.zip_filepath, mode="a", **self._zipfile_init_kw
             ) as zip_writer:
-                with zip_writer.open(k, **dict(self._open_kw, mode='w')) as fp:
+                with zip_writer.open(k, **dict(self._open_kw, mode="w")) as fp:
                     return fp.write(v)
 
     def __delitem__(self, k):
         try:
-            os.system(f'zip -d {self.zip_filepath} {k}')
+            os.system(f"zip -d {self.zip_filepath} {k}")
         except Exception as e:
-            raise KeyError(f'{e.__class__}: {e.args}')
-        # raise NotImplementedError("zipfile, the backend of ZipStore, doesn't support deletion,
+            raise KeyError(f"{e.__class__}: {e.args}")
+        # raise NotImplementedError("zipfile, the backend of ZipFiles, doesn't support deletion,
         # so neither will we.")
 
     def open(self):
-        self.zip_writer = ZipFile(self.zip_filepath, mode='a', **self._zipfile_init_kw)
+        self.zip_writer = ZipFile(self.zip_filepath, mode="a", **self._zipfile_init_kw)
         self.zip_writer_opened = True
         return self
 
@@ -855,22 +876,31 @@ class ZipStore(KvPersister):
         return False
 
 
+ZipStore = ZipFiles  # back-compatibility alias
 PathString = str
 PathFilterFunc = Callable[[PathString], bool]
 
 
+def _not_in(excluded, obj=None):
+    if obj is None:
+        return partial(_not_in, excluded)
+    return obj not in excluded
+
+
 def remove_some_entries_from_zip(
     zip_source,
-    keys_to_be_removed: Union[PathFilterFunc, Iterable[PathString]],
+    keys_to_be_removed: PathFilterFunc | Iterable[PathString],
     ask_before_before_deleting=True,
+    *,
+    remove_action: Literal["delete", "filter"] = "filter",
 ):
     """Removes specific keys from a zip file.
 
-    :param zip_source: zip filepath, bytes, or whatever a ``ZipStore`` can take
+    :param zip_source: zip filepath, bytes, or whatever a ``ZipFiles`` can take
     :param keys_to_be_removed: An iterable of keys or a boolean filter function
     :param ask_before_before_deleting: True (default) if the user should be
         presented with the keys first, and asked permission to delete.
-    :return: The ZipStore (in case you want to do further work with it)
+    :return: The ZipFiles (in case you want to do further work with it)
 
     Tip: If you want to delete with no questions asked, use currying:
 
@@ -881,7 +911,9 @@ def remove_some_entries_from_zip(
     ... )
 
     """
-    z = ZipStore(zip_source)
+    z = zip_source
+    if not isinstance(z, Mapping):
+        z = ZipFiles(z)
     if not isinstance(keys_to_be_removed, Callable):
         if isinstance(keys_to_be_removed, str):
             keys_to_be_removed = [keys_to_be_removed]
@@ -889,16 +921,19 @@ def remove_some_entries_from_zip(
         keys_to_be_removed = lambda x: x in set(keys_to_be_removed)
     keys_that_will_be_deleted = list(filter(keys_to_be_removed, z))
     if keys_that_will_be_deleted:
-        if ask_before_before_deleting:
-            print('These keys will be removed:\n\r')
-            print(*keys_that_will_be_deleted, sep='\n')
-            n = len(keys_that_will_be_deleted)
-            answer = input(f'\nShould I go ahead and delete these {n} keys? (y/N)')
-            if not answer == 'y':
-                print('Okay, I will NOT delete these.')
-                return
-    for k in keys_that_will_be_deleted:
-        del z[k]
+        if remove_action == "delete":
+            if ask_before_before_deleting:
+                print("These keys will be removed:\n\r")
+                print(*keys_that_will_be_deleted, sep="\n")
+                n = len(keys_that_will_be_deleted)
+                answer = input(f"\nShould I go ahead and delete these {n} keys? (y/N)")
+                if not answer == "y":
+                    print("Okay, I will NOT delete these.")
+                    return
+            for k in keys_that_will_be_deleted:
+                del z[k]
+        else:  # remove_action == 'filter'
+            z = filt_iter(z, filt=_not_in(keys_that_will_be_deleted))
 
     return z
 
@@ -915,7 +950,7 @@ remove_mac_junk_from_zip = partial(
     keys_to_be_removed=is_a_mac_junk_path,
     ask_before_before_deleting=False,
 )
-remove_mac_junk_from_zip.__doc__ = 'Removes mac junk keys from zip'
+remove_mac_junk_from_zip.__doc__ = "Removes mac junk keys from zip"
 
 # TODO: The way prefix and file_info_filt is handled is not efficient
 # TODO: prefix is silly: less general than filename_filt would be, and not even producing
@@ -930,8 +965,31 @@ remove_mac_junk_from_zip.__doc__ = 'Removes mac junk keys from zip'
 #                          name='ZipFileReader',
 #                          obj_of_data=ZipReader)
 
-#
-# if __name__ == '__main__':
-#     from dol.test.simple import test_local_file_ops
-#
-#     test_local_file_ops()
+
+# ----------------------------- Extras -------------------------------------------------
+
+
+def tar_compress(data_bytes, file_name="data.bin"):
+    import tarfile
+    import io
+
+    with io.BytesIO() as tar_buffer:
+        with tarfile.open(fileobj=tar_buffer, mode="w") as tar:
+            data_file = io.BytesIO(data_bytes)
+            tarinfo = tarfile.TarInfo(name=file_name)
+            tarinfo.size = len(data_bytes)
+            tar.addfile(tarinfo, fileobj=data_file)
+        return tar_buffer.getvalue()
+
+
+def tar_decompress(tar_bytes):
+    import tarfile
+    import io
+
+    with io.BytesIO(tar_bytes) as tar_buffer:
+        with tarfile.open(fileobj=tar_buffer, mode="r:") as tar:
+            for member in tar.getmembers():
+                extracted_file = tar.extractfile(member)
+                if extracted_file:
+                    return extracted_file.read()
+    return None
